@@ -18,7 +18,6 @@ from sciutil import SciUtil, SciException
 from scidat import Download
 from scidat import Annotate
 
-from functools import reduce
 import time
 from typing import Tuple
 import pandas as pd
@@ -37,7 +36,8 @@ class API:
     helpful interfaces that query between the data-structures.
     """
 
-    def __init__(self, manifest_file, gdc_client, clinical_file, sample_file, download_dir, meta_dir, requires_lst=None, clin_cols=None,
+    def __init__(self, manifest_file, gdc_client, clinical_file, sample_file, download_dir, meta_dir,
+                 annotation_file: str, requires_lst=None, clin_cols=None,
                  max_cnt=100, sciutil=None, split_manifest_dir=None, sep='_'):
         if gdc_client and '.exe' in gdc_client:
             self.file_type = 'windows'
@@ -59,7 +59,10 @@ class API:
         self.rna_df = None
         self.meth_df = None
         self.rna_meth_df = None
+        self.annotation_file = annotation_file
         self.setup_paths()
+        self.annotation_df = pd.read_csv(self.annotation_file)
+        self.annotation_df.set_index('ensembl_gene_id', inplace=True)
         self.download = Download(self.manifest_file, self.split_manifest_dir, self.download_dir, self.gdc_client,
                                  self.max_cnt, self.u)
         self.annotate = Annotate(self.meta_dir, self.clinical_file, self.sample_file, self.manifest_file,
@@ -71,7 +74,6 @@ class API:
         return change_str
 
     def setup_paths(self):
-
         self.meta_dir = self.add_windows_slash(self.meta_dir)
         self.download_dir = self.add_windows_slash(self.download_dir)
         self.split_manifest_dir = self.add_windows_slash(self.split_manifest_dir)
@@ -79,6 +81,7 @@ class API:
         self.clinical_file = self.add_windows_slash(self.clinical_file)
         self.sample_file = self.add_windows_slash(self.sample_file)
         self.manifest_file = self.add_windows_slash(self.manifest_file)
+        self.annotation_file = self.add_windows_slash(self.annotation_file)
 
     def download_data_from_manifest(self):
         self.download.download()
@@ -167,28 +170,16 @@ class API:
                 for g in df_tmp['Gene_Symbol'].values:
                     gene_symbols.append(g.split(';')[0])
 
-                df_tmp['gene_name'] = gene_symbols
-                df_tmp = df_tmp.fillna(0)
-                grouped_meth_df = df_tmp.groupby('gene_name')
-                gene_betas = []
-                gene_names = []
-                for g in grouped_meth_df:
-                    gene = g[0]
-                    beta_values = g[1]['Beta_value'].values
-                    beta_val = len(np.where(beta_values > beta_cutoff)[0]) / len(beta_values)
-                    gene_betas.append(beta_val)
-                    gene_names.append(gene)
-
-                    i += 1
-
                 meth_df = pd.DataFrame()
-                meth_df['id'] = gene_names
-                meth_df[f] = gene_betas
+                meth_df['Composite Element REF'] = df_tmp['Composite Element REF'].values
+                meth_df['external_gene_name'] = gene_symbols
+                meth_df['beta_value'] = df_tmp['Beta_value'].values
                 meth_df.to_csv(cpg_min_dir + f, index=False)
             except Exception as e:
                 self.u.err_p([f, e])
 
-    def build_meth_df(self, cpg_min_dir: str, df=None, drop_empty_rows=False, join_id='gene_id') -> None:
+    def build_meth_df(self, cpg_min_dir: str, df=None, drop_empty_rows=False, join_id='external_gene_name',
+                      meth_id='external_gene_name') -> None:
         """
         Here we build a dataframe based on the methylation data, we can add it to an existing dataframe
         and just join on the columns or create a new dataframe. Here we also choose whether to keep empty rows or only
@@ -216,26 +207,55 @@ class API:
                 cpg_files.append(f)
 
         start_idx = 0
-        if df is None and len(cpg_files) > 0:
-            df = pd.read_csv(cpg_min_dir + cpg_files[0])
-            name = self.annotate.annotated_file_dict[cpg_files[0]]['label']
-            df.columns = ['id', name]
-            start_idx = 1
-
+        files_parsed = []
+        cpg_to_genes = {}
+        cpg_to_beta = {}
         for f in cpg_files[start_idx:]:
             try:
                 df_tmp = pd.read_csv(cpg_min_dir + f)
                 name = self.annotate.annotated_file_dict[f]['label']
-                df_tmp.columns = ['id', name]
-
-                df = df.join(df_tmp.set_index('id'), on=join_id)
+                df_tmp.columns = ['Composite Element REF', meth_id, name]
+                gene_names = df_tmp[meth_id].values
+                beta_values = df_tmp[name].values
+                # Build dict from each one
+                for i, cpg in enumerate(df_tmp['Composite Element REF'].values):
+                    # Don't keep cpgs that don't have gene names
+                    if gene_names[i] != '.':
+                        if cpg_to_genes.get(cpg) == None:
+                            cpg_to_genes[cpg] = gene_names[i]
+                            cpg_to_beta[cpg] = {}
+                        if cpg_to_genes.get(cpg) != gene_names[i]:
+                            self.u.warn_p(['WARN mismatch between gene names for same CpG: ', cpg, gene_names[i],
+                                           cpg_to_genes.get(cpg)])
+                        cpg_to_beta[cpg][name] = beta_values[i]
+                files_parsed.append(name)
             except Exception as e:
-                print(e)
+                self.u.warn_p(['WARNING: minify meth files, unable to parse: ', f,
+                               '\nIs this file missisng from your sample or clinical file?'])
+        df = pd.DataFrame()
+        # Now we want to build the df with each of our columns for the beta values and the gene names should be
+        # always the same
+        f_betas = {}
+        gene_names = []
+        cpgs = []
+        for cpg, beta_vals in cpg_to_beta.items():
+            for f in files_parsed:
+                if not f_betas.get(f):
+                    f_betas[f] = []
+                f_betas[f].append(beta_vals.get(f))
+            gene_names.append(cpg_to_genes.get(cpg))
+            cpgs.append(cpg)
+        df['cpg-id'] = cpgs
+        df[join_id] = gene_names
+        for f, vals in f_betas.items():
+            df[f] = vals
+
         if drop_empty_rows:
             df = df.dropna()
         self.meth_df = df
 
-    def merge_rna_meth_values(self, meth_df: pd.DataFrame, rna_df: pd.DataFrame, index_col='id', merge_col='gene_id') -> pd.DataFrame:
+    def merge_rna_meth_values(self, meth_df: pd.DataFrame, rna_df: pd.DataFrame, index_col='external_gene_name',
+                              merge_col='external_gene_name') -> pd.DataFrame:
         self.rna_meth_df = rna_df.join(meth_df.set_index(index_col), on=merge_col)
         return self.rna_meth_df
 
@@ -593,4 +613,21 @@ class API:
         if case_ids is not None:
             return self.rna_meth_df[self.rna_meth_df['case_id'].isin(case_ids)]
 
-        return self.rna_meth_df
\ No newline at end of file
+        return self.rna_meth_df
+
+    def add_gene_metadata_to_df(self, df:pd.DataFrame, left_id='id', right_id='ensembl_gene_id'):
+        """ Adds gene name information to the RNAseq dataframe """
+        # We need to remove the '.' from the ensembl ids
+        ensembl_ids = []
+        for g in df[left_id].values:
+            ensembl_ids.append(g.split('.')[0])
+
+        df[right_id] = ensembl_ids
+        df.set_index(right_id, inplace=True)
+
+        joined_df = df.join(self.annotation_df, how='left', on=right_id)
+        return joined_df
+
+    def set_gene_annotation_file(self, filepath):
+        """ Set the annotation file to override the default. """
+        self.annotation_df = pd.read_csv(filepath)
\ No newline at end of file
